@@ -32,10 +32,11 @@
 // Copyright (C) 2010 William Bader <william@newspapersystems.com>
 // Copyright (C) 2021 Mahmoud Khalil <mahmoudkhalil11@gmail.com>
 // Copyright (C) 2021 Georgiy Sgibnev <georgiy@sgibnev.com>. Work sponsored by lab50.net.
-// Copyright (C) 2023 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2023, 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 // Copyright (C) 2023 Ilaï Deutel <idtl@google.com>
 // Copyright (C) 2023 Even Rouault <even.rouault@spatialys.com>
 // Copyright (C) 2024 Nelson Benítez León <nbenitezl@gmail.com>
+// Copyright (C) 2024 Vincent Lefevre <vincent@vinc17.net>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -161,12 +162,14 @@ ObjectStream::ObjectStream(XRef *xref, int objStrNumA, int recursion)
         error(errSyntaxError, -1, "Too many objects in an object stream");
         return;
     }
+    if (!objStr.streamReset()) {
+        return;
+    }
     objs = new Object[nObjects];
     objNums = (int *)gmallocn(nObjects, sizeof(int));
     offsets = (Goffset *)gmallocn(nObjects, sizeof(Goffset));
 
     // parse the header: object numbers and offsets
-    objStr.streamReset();
     str = new EmbedStream(objStr.getStream(), Object(objNull), true, first);
     parser = new Parser(xref, str, false);
     for (i = 0; i < nObjects; ++i) {
@@ -257,6 +260,7 @@ XRef::XRef() : objStrs { 5 }
     permFlags = defPermFlags;
     ownerPasswordOk = false;
     rootNum = -1;
+    rootGen = -1;
     strOwner = false;
     xrefReconstructed = false;
     encAlgorithm = cryptNone;
@@ -746,7 +750,10 @@ bool XRef::readXRefStream(Stream *xrefStr, Goffset *pos)
         return false;
     }
 
-    xrefStr->reset();
+    if (!xrefStr->reset()) {
+        return false;
+    }
+
     const Object &idx = dict->lookupNF("Index");
     if (idx.isArray()) {
         for (int i = 0; i + 1 < idx.arrayGetLength(); i += 2) {
@@ -905,7 +912,9 @@ bool XRef::constructXRef(bool *wasReconstructed, bool needCatalogDict)
         xrefReconstructedCb();
     }
 
-    str->reset();
+    if (!str->reset()) {
+        return false;
+    }
     while (true) {
         pos = str->getPos();
         if (!str->getLine(buf, 256)) {
@@ -1289,13 +1298,13 @@ err:
             }
         }
         if (xrefHasChanges) {
-            error(errInternal, -1, "xref num {0:d} not found but needed, document has changes, reconstruct aborted\n", num);
+            error(errInternal, -1, "xref num {0:d} not found but needed, document has changes, reconstruct aborted", num);
             // pretend we constructed the xref, otherwise we will do this check again and again
             xrefReconstructed = true;
             return Object(objNull);
         }
 
-        error(errInternal, -1, "xref num {0:d} not found but needed, try to reconstruct\n", num);
+        error(errInternal, -1, "xref num {0:d} not found but needed, try to reconstruct", num);
         rootNum = -1;
         constructXRef(&xrefReconstructed);
         return fetch(num, gen, ++recursion, endPos);
@@ -1448,12 +1457,12 @@ void XRef::setModifiedObject(const Object *o, Ref r)
 {
     xrefLocker();
     if (r.num < 0 || r.num >= size) {
-        error(errInternal, -1, "XRef::setModifiedObject on unknown ref: {0:d}, {1:d}\n", r.num, r.gen);
+        error(errInternal, -1, "XRef::setModifiedObject on unknown ref: {0:d}, {1:d}", r.num, r.gen);
         return;
     }
     XRefEntry *e = getEntry(r.num);
     if (unlikely(e->type == xrefEntryFree)) {
-        error(errInternal, -1, "XRef::setModifiedObject on ref: {0:d}, {1:d} that is marked as free. This will cause a memory leak\n", r.num, r.gen);
+        error(errInternal, -1, "XRef::setModifiedObject on ref: {0:d}, {1:d} that is marked as free. This will cause a memory leak", r.num, r.gen);
     }
     e->obj = o->copy();
     e->setFlag(XRefEntry::Updated, true);
@@ -1496,7 +1505,7 @@ void XRef::removeIndirectObject(Ref r)
 {
     xrefLocker();
     if (r.num < 0 || r.num >= size) {
-        error(errInternal, -1, "XRef::removeIndirectObject on unknown ref: {0:d}, {1:d}\n", r.num, r.gen);
+        error(errInternal, -1, "XRef::removeIndirectObject on unknown ref: {0:d}, {1:d}", r.num, r.gen);
         return;
     }
     XRefEntry *e = getEntry(r.num);
@@ -1512,10 +1521,10 @@ void XRef::removeIndirectObject(Ref r)
     setModified();
 }
 
-Ref XRef::addStreamObject(Dict *dict, char *buffer, const Goffset bufferSize, StreamCompression compression)
+Ref XRef::addStreamObject(Dict *dict, std::vector<char> buffer, StreamCompression compression)
 {
-    dict->add("Length", Object((int)bufferSize));
-    AutoFreeMemStream *stream = new AutoFreeMemStream(buffer, 0, bufferSize, Object(dict));
+    dict->add("Length", Object((int)buffer.size()));
+    AutoFreeMemStream *stream = new AutoFreeMemStream(std::move(buffer), Object(dict));
     stream->setFilterRemovalForbidden(true);
     switch (compression) {
     case StreamCompression::None:;
@@ -1527,16 +1536,11 @@ Ref XRef::addStreamObject(Dict *dict, char *buffer, const Goffset bufferSize, St
     return addIndirectObject(Object((Stream *)stream));
 }
 
-Ref XRef::addStreamObject(Dict *dict, uint8_t *buffer, const Goffset bufferSize, StreamCompression compression)
-{
-    return addStreamObject(dict, (char *)buffer, bufferSize, compression);
-}
-
 void XRef::writeXRef(XRef::XRefWriter *writer, bool writeAllEntries)
 {
     // create free entries linked-list
     if (getEntry(0)->gen != 65535) {
-        error(errInternal, -1, "XRef::writeXRef, entry 0 of the XRef is invalid (gen != 65535)\n");
+        error(errInternal, -1, "XRef::writeXRef, entry 0 of the XRef is invalid (gen != 65535)");
     }
     int lastFreeEntry = 0;
     for (int i = 0; i < size; i++) {
